//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     flist.h
// Purpose:    Lists of Formulas
// Last Change:29.03.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	FLIST_INCLUDED
#define FLIST_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef FML_INCLUDED
#include "fml.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// FLIST_MAGIC: Magic number (identifies objects of this class for debugging).
//-----------------------------------------------------------------------------

#ifdef VER_DEBUG
static const long FLIST_MAGIC =	0x464C4953L;	// 'FLIS'
#endif

//=============================================================================
// Class for Lists of Terms:
//=============================================================================

class flist_c {

	public:

//-----------------------------------------------------------------------------
// Methods:
//-----------------------------------------------------------------------------

	// Constructor:
	flist_c(fml_c* head);

	// Destructor:
	~flist_c(void);

	// Head of the list:
	inline fml_c*	head(void)
		{ CHECK(valid(), "flist_c::head: Invalid Object");
		  return(Head); }

	// Tail of the list:
	inline flist_c*	tail(void)
		{ CHECK(valid(), "flist_c::tail: Invalid Object");
		  return(Tail); }

	// Append a list (only possible if tail empty):
	void append(flist_c* tail);

	// Print this formula list:
	void print(void);

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity Check:
	public:
	bool_t	valid(void);

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "FLIST_MAGIC"
#endif

//-----------------------------------------------------------------------------
// Private Members:
//-----------------------------------------------------------------------------

	private:

	// Head: First term in the list.
	fml_c* 	Head;

	// Tail: Rest of the list (i.e. this list except the first term).
	flist_c* Tail;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef flist_c *flist_t;

//=============================================================================
// End of Include File:
//=============================================================================

#endif

